package Parser;

use strict;
use warnings;
use vars qw(%datafields %extra @EXPORT_OK);

use POSIX;
use Exporter;

@EXPORT_OK = qw#parse extra %datafields#;

@Parser::ISA = ('Exporter');

$Parser::NAME = 'squid';
$Parser::VERSION = '0.1.0';
$Parser::AUTHOR = 'Florian octo Forster';
$Parser::EMAIL = 'octo@verplant.org';

print "\nparser/squid: Using squid access log format." if $::DEBUG;
#
# label => 'class:type'
# class key is used to index data (was cntkey);
# class amount is used to aggregate (was sumkey);
# labels are used by report and is a subject to translate;
# types are used by report to format values;
#
%datafields = (
    'date'	    => 'key:date',
    'hour'	    => 'key:time',
    'client'	    => 'key:host',
    'server'	    => 'key:host',
    'peer'	    => 'key:host',
    'protocol'	    => 'key',
    'method'	    => 'key',
    'mime'	    => 'key',
    'httpstatus'    => 'key',
    'resultcode'    => 'key',
    'hierarchycode' => 'key',
    'bytes'	    => 'amount:bytes',
    'elapsed'	    => 'amount:duration',
    'requests'	    => 'amount:number',
    );

# year 2038 unsafe
%extra = (
    'start'=>(2**31-1),
    'end'=>0,
    'total'=>0,
    'days'=>[],
    );

# This needs to be done at runtime, since Data uses Setup which relies on
# %datafields to be defined  -octo
require Data;
import Data qw#store#;

return (1);
    
sub parse
{
	my $line = shift or return undef;
	if ($line =~ /^(\S+)\s+(\d+)\s(\S+)\s(.+?)\/(\d+)\s(\d+)\s(\S+)\s(\S+)\s(\S+)\s(.+?)\/(\S+)\s(.*)$/)
	{
		my ($epoch, $duration, $client, $result_code, $http_code,
			$size, $method, $url, $ident, $hierarchy_code,
			$origin_host, $mime) = 
		($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12);

		return undef if ($url eq '*');

		my @timestamp = (localtime ($epoch));
		# both date and time' are in seconds
		my $date = POSIX::mktime(0,0,0,@timestamp[3..5]);
		my $hour = $timestamp[2]*3600;

		$extra{'start'}=$epoch if( $epoch < $extra{'start'} );
		$extra{'end'}=$epoch   if( $epoch > $extra{'end'} );
		$extra{'total'}++;
		push (@{$extra{'days'}}, $date)
		    unless (grep { $_ eq $date } @{$extra{'days'}});

		my ($protocol, $server, $path);
		if ($url =~ /^(\w+?)\:\/\/(.+?)(?:\:\d+)?(\/.*?)\??$/)
		{
			($protocol, $server, $path) = ($1, $2, $3);
		}
		elsif ($url =~ /^([\w\d\-\.]+):443$/)
		{
			($protocol, $server, $path) = ('https', $1, '');
		}
		else
		{
			return undef;
		}

		if ($server =~ /\.[a-z]{2,3}$/ and $::config->{'referer'}{'host_only'})
		{
			$server =~ s/^.+\.(.+?\..+?)$/$1/;
		}
		elsif ($server =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ and $::config->{'referer'}{'host_only'})
		{
			$server = 'UNRESOLVED';
		}
		
		my %combined=(
			    'client'	    => $client,
			    'resultcode'    => uc ($result_code),
			    'httpstatus'    => $http_code,
			    'method'	    => $method,
			    'mime'	    => $mime,
			    'bytes'	    => $size,
			    'server'	    => $server,
			    'protocol'	    => uc ($protocol),
			    'hierarchycode' => uc ($hierarchy_code),
			    'peer'	    => $origin_host,
			    'date'	    => $date,
			    'hour'	    => $hour,
			    'elapsed'	    => $duration,
			    'requests'	    => 1,
			    );
		store (\%combined);
		return 1;
	}
	return undef;
}

sub extra 
{
    my $start = POSIX::mktime(0,0,0,(localtime($extra{'start'}))[3..5]);
    my $end = POSIX::mktime(0,0,0,(localtime($extra{'end'}))[3..5]);
    my $days=scalar @{$extra{'days'}};
    my $average=int($extra{'total'}/$days);
    
    $::extra->{'Reporting days'} = $days;
    $::extra->{'Total requests'} = $extra{'total'};
    $::extra->{'Averange requests per day'} = $average;
}
